package it.univr.prog.bioinfo.pizzeria;

import it.univr.prog.bioinfo.pizzeria.eccezioni.IngredienteEsistente;
import it.univr.prog.bioinfo.pizzeria.eccezioni.IngredienteNonEsistente;
import it.univr.prog.bioinfo.pizzeria.eccezioni.OrdineNonEsistente;
import it.univr.prog.bioinfo.pizzeria.eccezioni.PizzaEsistente;
import it.univr.prog.bioinfo.pizzeria.eccezioni.PizzaNonEsistente;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Classe oggetto per la gestione e la vendita delle pizze.
 * 
 * @author Damiano Macedonio
 * @version 2.0
 */

public class Pizzeria {

	/**
	 * Menu associato alla pizzeria, contiene tutte le pizze offerte.
	 */
	private Map<String, Pizza> menu;

	/**
	 * Ordini ricevuti dalla pizzeria.
	 */
	private List<Ordine> ordini;

	/**
	 * Contiene gli ingredienti disponibili per la pizzeria.
	 */
	private Map<String, Ingrediente> dispensa;

	/**
	 * Crea un nuovo oggetto Pizzeria con parametri impostati dall'utente.
	 */
	public Pizzeria() {
		menu = new HashMap<String, Pizza>();
		ordini = new ArrayList<Ordine>();
		dispensa = new HashMap<String, Ingrediente>();
	}

	/**
	 * Aggiunge un ingrediente a <code>dispensa</code>. Lancia <code>IngredienteEsistente</code>
	 * se l'igrediente e' gia' presente.
	 * 
	 * @param nome
	 *            identificativo dell'ingrediente
	 * @param descrizione
	 *            caratteristiche dell'ingrediente
	 * @param prezzo
	 *            cifra da aggiungere al prezzo della pizza in cui compare
	 *            l'ingrediente
	 * @exception IngredienteEsistente
	 *                se l'ingrediente e' gia' presente in <code>dispensa</code>
	 */
	public void aggiungiIngrediente(String nome, String descrizione,
			double prezzo) throws IngredienteEsistente {
		if (dispensa.containsKey(nome)) {
			throw new IngredienteEsistente("ingrediente " + nome
					+ " gia' presente in dispensa");
		}
		dispensa.put(nome, new Ingrediente(nome, descrizione, prezzo));
	}

	/**
	 * Cerca un ingrediente in <code>dispensa</code>.
	 * 
	 * @param ingrediente
	 *            nome dell'ingrediente da cercare
	 * @return l'ingrediente se presente
	 * @exception IngredienteNonEsistente
	 *                se l'ingrediente non e' presente in <code>dispensa</code>
	 */
	public Ingrediente getIngrediente(String ingrediente)
			throws IngredienteNonEsistente {
		Ingrediente i = dispensa.get(ingrediente);
		if (i == null)
			throw new IngredienteNonEsistente("ingrediente " + ingrediente
					+ " non presente in dispensa.");
		return i;
	}

	/**
	 * Aggiunge una nuova pizza a <code>menu</code>. La pizza viene aggiunta
	 * solo se non ancora presente in <code>menu</code> e tutti gli ingredienti
	 * che la compongono sono presenti in <code>dispensa</code>.
	 * 
	 * @param nome
	 *            identificativo della nuova pizza
	 * @param ingredienti
	 *            ingredienti che compongono la pizza
	 * @exception PizzaEsistente
	 *                se la pizza e' presente in <code>menu</code>
	 * @exception IngredienteNonEsistente
	 *                se uno degli ingredienti non e' presente in
	 *                <code>dispensa</code>
	 */
	public void aggiungiPizza(String nome, String... ingredienti)
			throws PizzaEsistente, IngredienteNonEsistente {
		if (menu.containsKey(nome))
			throw new PizzaEsistente("pizza " + nome + " gia' presente in menu.");
		List<Ingrediente> listaIngredienti = new ArrayList<Ingrediente>();
		for (String i : ingredienti)
			listaIngredienti.add(getIngrediente(i));
		Pizza p = new Pizza(nome, listaIngredienti);
		menu.put(nome, p);
	}

	/**
	 * Cerca una pizza in <code>menu</code>.
	 * 
	 * @param nome
	 *            identificativo della pizza da cercare Il valore deve essere
	 *            diverso da <code>null</code>
	 * @return l'oggetto <code>Pizza</code> corrispondente se presente in
	 *         <code>menu</code>, <code>null</code> altrimenti
	 * @throws PizzaNonEsistente
	 *             se la pizza non è presente in menu
	 */
	public Pizza getPizza(String nome) throws PizzaNonEsistente {
		Pizza p = menu.get(nome);
		if (p == null)
			throw new PizzaNonEsistente("pizza " + nome
					+ " non presente in menu.");
		return p;
	}

	/**
	 * Genera l'elenco delle pizze presenti in <code>menu</code>.
	 * 
	 * @return l'elenco delle pizze in <code>menu</code>
	 */
	public String menuPizze() {
		String str = "";
		for (Pizza p : menu.values())
			str += "\n" + p.toString() + "\n";
		return str;
	}

	/**
	 * Aggiunge a <code>ordini</code> un nuovo ordine con le pizze individuate
	 * dagli identificativi passati. L'ordine viene aggiunto se tutte le
	 * pizze richieste compaiono in <code>menu</code>.
	 * 
	 * @param pizze
	 *            identificativi delle pizze da aggiungere
	 * @return il numero dell'ordine se l'inserimento ha avuto successo
	 * @throws PizzaNonEsistente
	 *             se una delle pizze non compare in <code>menu</code>
	 */
	public int ordinaPizze(String... pizze) throws PizzaNonEsistente {
		List<Pizza> listaPizze = new ArrayList<Pizza>();
		for (String p : pizze)
			listaPizze.add(getPizza(p));
		Ordine o = new Ordine(listaPizze);
		ordini.add(o);
		return ordini.indexOf(o);
	}

	/**
	 * Genera lo scontrino relativo al <code>numeroOrdine</code>.
	 * 
	 * @param numeroOrdine
	 *            indetificativo dell'ordine
	 * @return lo scontrino relativo se l'ordine e' presente, <code>null</code>
	 *         altrimenti
	 * @throws OrdineNonEsistente
	 *             se l'ordine non e' presente
	 */
	public String generaScontrino(int numeroOrdine) throws OrdineNonEsistente {
		try {
			return this.ordini.get(numeroOrdine).toString();
		} catch (IndexOutOfBoundsException e) {
			throw new OrdineNonEsistente("Ordine numero " + numeroOrdine
					+ " non esistente");
		}
	}

	/**
	 * Genera l'elenco degli ingredienti in <code>dispensa</code>.
	 * 
	 * @return l'elenco degli ingredienti in <code>dispensa</code>
	 */
	public String elencoIngredienti() {
		String str = "";
		for (Ingrediente i : dispensa.values())
			str += i.getNome() + "\n";
		return str;
	}

}